key in dict.keys() # SIM118

foo["bar"] in dict.keys() # SIM118

foo() in dict.keys() # SIM118

for key in dict.keys(): # SIM118
    pass

for key in list(dict.keys()):
    if some_property(key):
        del dict[key]



"""Test that a global import which is redefined locally, but used later in another scope does not generate a warning."""

import unittest, transport


class GetTransportTestCase(unittest.TestCase):
    def test_get_transport(self):
        transport = 'transport'
        self.assertIsNotNone(transport)


class TestTransportMethodArgs(unittest.TestCase):
    def test_send_defaults(self):
        transport.Transport()



# Replace names by built-in names, whether namespaced or not
# https://github.com/search?q=%22from+six+import%22&type=code
import six
from six.moves import map  # No need
from six import text_type

six.text_type  # str
six.binary_type  # bytes
six.class_types  # (type,)
six.string_types  # (str,)
six.integer_types  # (int,)
six.unichr  # chr
six.iterbytes  # iter
six.print_(...)  # print(...)
six.exec_(c, g, l)  # exec(c, g, l)
six.advance_iterator(it)  # next(it)
six.next(it)  # next(it)
six.callable(x)  # callable(x)
six.moves.range(x)  # range(x)
six.moves.xrange(x)  # range(x)
isinstance(..., six.class_types)  # isinstance(..., type)
issubclass(..., six.integer_types)  # issubclass(..., int)
isinstance(..., six.string_types)  # isinstance(..., str)

# Replace call on arg by method call on arg
six.iteritems(dct)  # dct.items()
six.iterkeys(dct)  # dct.keys()
six.itervalues(dct)  # dct.values()
six.viewitems(dct)  # dct.items()
six.viewkeys(dct)  # dct.keys()
six.viewvalues(dct)  # dct.values()
six.assertCountEqual(self, a1, a2)  # self.assertCountEqual(a1, a2)
six.assertRaisesRegex(self, e, r, fn)  # self.assertRaisesRegex(e, r, fn)
six.assertRegex(self, s, r)  # self.assertRegex(s, r)

# Replace call on arg by arg attribute
six.get_method_function(meth)  # meth.__func__
six.get_method_self(meth)  # meth.__self__
six.get_function_closure(fn)  # fn.__closure__
six.get_function_code(fn)  # fn.__code__
six.get_function_defaults(fn)  # fn.__defaults__
six.get_function_globals(fn)  # fn.__globals__

# Replace by string literal
six.b("...")  # b'...'
six.u("...")  # '...'
six.ensure_binary("...")  # b'...'
six.ensure_str("...")  # '...'
six.ensure_text("...")  # '...'
six.b(string)  # no change

# Replace by simple expression
six.get_unbound_function(meth)  # meth
six.create_unbound_method(fn, cls)  # fn

# Raise exception
six.raise_from(exc, exc_from)  # raise exc from exc_from
six.reraise(tp, exc, tb)  # raise exc.with_traceback(tb)
six.reraise(*sys.exc_info())  # raise

# Int / Bytes conversion
six.byte2int(bs)  # bs[0]
six.indexbytes(bs, i)  # bs[i]
six.int2byte(i)  # bytes((i, ))

# Special cases for next calls
next(six.iteritems(dct))  # next(iter(dct.items()))
next(six.iterkeys(dct))  # next(iter(dct.keys()))
next(six.itervalues(dct))  # next(iter(dct.values()))

# TODO: To implement


# Rewrite classes
@six.python_2_unicode_compatible  # Remove
class C(six.Iterator):
    pass  # class C: pass


class C(six.with_metaclass(M, B)):
    pass  # class C(B, metaclass=M): pass


# class C(B, metaclass=M): pass
@six.add_metaclass(M)
class C(B):
    pass



def f(a, b):
    print("Hello, world!")


def f(a, b, *args, **kwargs):
    print("Hello, world!")


class C:
    def f(self, a, b):
        print("Hello, world!")

    def f(self, a, b, *args, **kwargs):
        print("Hello, world!")



x = 1  # type: ignore
x = 1  # type ignore
x = 1  # type:ignore

x = 1
x = 1  # type ignore  # noqa
x = 1  # type: ignore[attr-defined]
x = 1  # type: ignore[attr-defined, name-defined]
x = 1  # type: ignore[type-mismatch]  # noqa
x = 1  # type: Union[int, str]
x = 1  # type: ignoreme




if True:
    import foo; x = 1
    import foo;     x = 1

if True:
    import foo; \
x = 1

if True:
    import foo \
        ; x = 1


if True:
    x = 1; import foo


if True:
    x = 1; \
         import foo


if True:
    x = 1 \
        ; import foo


if True:
    x = 1; import foo; x = 1
    x = 1;     import foo;     x = 1

if True:
    x = 1; \
        import foo; \
    x = 1

if True:
    x = 1 \
        ;import foo \
        ;x = 1


# Continuation, but not as the last content in the file.
x = 1; \
import foo

# Continuation, followed by end-of-file. (Removing `import foo` would cause a syntax
# error.)
x = 1; \
import foo